/* $Id: vga_io.inc,v 1.9 1999/08/13 14:34:41 becka Exp $
******************************************************************************

   IBM VGA I/O

   Copyright (C) 1996-1997 Jason McMullan	[jmcc@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

   This code was derived from the following sources of information:
   [1] Xfree86 VGA driver

******************************************************************************
*/

/* GGI includes */

#include "IBM/vga_io.inc"

/* Generalized DAC output functions */

static kgiu16 dac_port[4] =
{ 
	0x3C8, 0x3C9, 0x3C6, 0x3C7 
};

void DAC_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 dac_reg)
{
	if (dac_reg & ~0x3) 
	{
		
		DEBUG1("DAC register select lines truncated: %x to %x",
		       dac_reg, dac_reg &= 0x3);
	}
	
	io_out8(value, dac_port[dac_reg & 0x03]);
}

kgiu8 DAC_in8(struct kgi_display *dpy, kgiu8 dac_reg)
{
	if(dac_reg & ~0x03) 
	{
		
		DEBUG1("DAC register select lines truncated: %x to %x",
		       dac_reg, dac_reg &= 0x03);
	}
	
	return io_in8(dac_port[dac_reg & 0x03]);
 }

void DAC_ins8(struct kgi_display *dpy,kgiu8 dac_reg,void *buf, kgi_uint count)
{
	if(dac_reg & ~0x03)
	{
		
		DEBUG1("DAC register select lines truncated: %x to %x",
		       dac_reg, dac_reg &= 0x3);
	}

	io_ins8(dac_port[dac_reg & 0x03], buf, count);
}

void DAC_outs8(struct kgi_display *dpy,kgiu8 dac_reg, void *buf, kgi_uint count)
{
	if(dac_reg & ~0x03) 
	{
		
		DEBUG1("DAC register select lines truncated: %x to %x",
		       dac_reg, dac_reg &= 0x03);
	}

	io_outs8(dac_port[dac_reg & 0x03], buf, count);
}

/* Clock I/O functions */

void CLK_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 reg)
{
	if (reg != 0)
	{
		
		INTERNAL_ERROR;
	}
	
	switch (value) {
	case 0: 
		value = MISC_25MHZ_CLK;
		break;
	case 1:
		value = MISC_28MHZ_CLK;
		break;
	default:
		INTERNAL_ERROR;
		return;
	}

	io_out8((io_in8(MISCr) & 0xf3) | value, MISCw);
}

kgiu8 CLK_in8(struct kgi_display *dpy,kgiu8 reg)
{
	if (reg != 0)
	{
		
		INTERNAL_ERROR;
	}
	

	return (io_in8(MISCr) & 0x04) >> 2;
}

#define NoSEQRegs	0x05
#define	NoCRTRegs	0x19
#define NoGRCRegs	0x09
#define NoATCRegs	0x15

struct statetable
{
	kgiu8 MISC;
	kgiu8 SEQ[NoSEQRegs];
	kgiu8 CRT[NoCRTRegs];
	kgiu8 GRC[NoGRCRegs];
	kgiu8 ATC[NoATCRegs];
};

static void save_SEQ(kgiu8 *buf)
{
	int i;

	for (i = 0; i < NoSEQRegs; i++) 
	{
		
		*(buf++) = inbSEQ(i);
	}
}

static void restore_SEQ(kgiu8 *buf)
{
	register kgiu8 i;
	for (i = 0; i < NoSEQRegs; i++) 
	{
		
		outbSEQ(*(buf++), i);
	}
}

static void save_CRT(kgiu8 *buf)
{
	int i;

	for (i = 0; i < NoCRTRegs; i++) 
	{
		
		*(buf++) = inbCRT(i);
	}
}

static void restore_CRT(kgiu8 *buf)
{
	register kgiu8 i;

	outbCRT(inbCRT(CRT_VSYNCEND) & ~CR11_LOCKTIMING, CRT_VSYNCEND);
	outbCRT(inbCRT(CRT_HBLANKEND) | CR03_IS_VGA,CRT_HBLANKEND);

	for (i = 0; i < NoCRTRegs; i++) 
	{
		switch (i) 
		{
		case CRT_HBLANKEND:
			*buf |= CR03_IS_VGA;
			break;
		case CRT_VSYNCEND:
			*buf &= ~CR11_LOCKTIMING;
			break;
		default:
			break;
		}
		outbCRT(*(buf++), i);
	}
	
}

static void save_GRC(kgiu8 *buf)
{
  int i;

	for (i = 0; i < NoGRCRegs; i++) 
	{
		
		*(buf++) = inbGRC(i);
	}
}

static void restore_GRC(kgiu8 *buf)
{
	register kgiu8 i;

	for (i = 0; i < NoGRCRegs; i++) 
		outbGRC(*(buf++), i);
}

static void save_ATC(kgiu8 *buf)
{
	int i;

	for (i = 0; i < NoATCRegs; i++) 
	{
		io_in8(INS1r);  /* Reset AFF and setup index */
		*(buf++) = inbATC(i);
	}
	io_in8(INS1r); 
	io_out8(ATCI_ENABLEDISPLAY, ATCI);
}

static void restore_ATC(kgiu8 *buf)
{
	register kgiu8 i;

	for (i = 0; i < NoATCRegs; i++) {
		io_in8(INS1r); 
		outbATC(*(buf++), i);
	}
	io_in8(INS1r); 
	io_out8(ATCI_ENABLEDISPLAY, ATCI);
}

static void save_state(struct statetable *state) 
{
	state->MISC = io_in8(MISCr);	/* Save miscellaneous output reg */
	
	outbMISC(state->MISC);  /* Adjust IO base */
	save_SEQ(state->SEQ);	/* Save sequencer regs */
	save_CRT(state->CRT);	/* Save CRT controller regs */
	save_GRC(state->GRC);	/* Save graphics controller regs */
	save_ATC(state->ATC);	/* Save attribute controller regs */
}

static void restore_state(struct statetable *state)
{
	outbMISC((io_in8(MISCr) & MISC_CLOCK_MASK) | state->MISC);

	restore_SEQ(state->SEQ);
	restore_CRT(state->CRT);
	restore_GRC(state->GRC);
	restore_ATC(state->ATC);
}

/* FIXME: Move this over with the other KGI debugging stuff */
#if DEBUG_LEVEL > 0
static void dump_state(struct statetable *state)
{ 
	int i;
	kgi_printk("MISC: %.2x", state->MISC);
  
	for (i = 0; i < NoSEQRegs; i++)
	{ 
		if(!(i & 0xF))
			kgi_printk("\nSEQ:%.2x: ", i);
		kgi_printk("%.2x, ", state->SEQ[i]);
	}
	
	for (i = 0; i < NoCRTRegs; i++)
	{ 
		if(!(i & 0xf))
			kgi_printk("\nCRT:%.2x: ", i);
		kgi_printk("%.2x, ", state->CRT[i]);
	}
	
	for (i = 0; i < NoGRCRegs; i++)
	{ 
		if(!(i & 0xF)) 
			kgi_printk("\nGRC:%.2x: ", i);
		kgi_printk("%.2x, ", state->GRC[i]);
	}
	
	for (i = 0; i < NoATCRegs; i++)
	{ 
		if(!(i & 0xF)) 
			kgi_printk("\nATC:%.2x: ", i);
		kgi_printk("%.2x, ", state->ATC[i]);
	}
	kgi_printk("\n");
}
#endif
